package cn.com.taiji.tile.service.impl;

import cn.com.taiji.common.domain.PageResult;
import cn.com.taiji.common.model.LayerGroupDsView;
import cn.com.taiji.common.model.LayerStyleView;
import cn.com.taiji.cql.model.GeoRequestParam;
import cn.com.taiji.cql.service.IECqlService;
import cn.com.taiji.tile.model.POI;
import cn.com.taiji.tile.service.ITileService;
import cn.com.taiji.tile.style.ShapeStyle;
import cn.com.taiji.tile.util.ShapeUtils;
import lombok.extern.slf4j.Slf4j;
import org.geotools.jdbc.JDBCDataStoreFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.imageio.ImageIO;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.*;
import java.util.List;

@Slf4j
@Service
public class TileServiceImpl implements ITileService {

    @Autowired
    private IECqlService ecqlService;


    @Override
    public byte[] tileData(GeoRequestParam requestParam, List<LayerGroupDsView> layerGroupDsViewList, Map<String, List<LayerStyleView>> dataMaps) {
        BufferedImage image = new BufferedImage(256, 256, BufferedImage.TYPE_INT_ARGB);
        Graphics2D g2 = image.createGraphics();
        for (LayerGroupDsView layerGroupDsView : layerGroupDsViewList) {


            //获取图层代码对应的样式
            List<LayerStyleView> layerStyleViewList = new ArrayList<LayerStyleView>();

            if (dataMaps.containsKey(layerGroupDsView.getLayerCode())) {

                layerStyleViewList = dataMaps.get(layerGroupDsView.getLayerCode());

            }


            //2根据cql规则获取图层对应的空间地理数据

            long currentTimeMillis = System.currentTimeMillis();

            StringBuffer filterCqlStr = new StringBuffer();

            String filterCql = requestParam.getCqlFilter();

            if (filterCql != null && !"".equals(filterCql)) {

                filterCqlStr.append(filterCql);
            }

            String bbox = requestParam.getBbox();
            if(bbox !=null && !"".equals(bbox)){

                StringBuffer sbBbox = new StringBuffer();
                sbBbox.append("(bbox (");
                sbBbox.append(layerGroupDsView.getBoxColumn()+",");
                sbBbox.append(requestParam.getBbox()+"))");

                if(filterCqlStr.length() >0){

                    filterCqlStr.append(" and "+sbBbox.toString());
                }else{

                    filterCqlStr.append(sbBbox.toString());

                }

            }

            String layerDataType = layerGroupDsView.getLayerDataType();
            String layerDataRun = layerGroupDsView.getLayerDataRun();

            if (layerDataType != null && !"".equals(layerDataType) && "cql".equals(layerDataType)) {

                if (layerDataRun != null && !"".equals(layerDataRun)) {

                    if (filterCqlStr.length() > 0) {

                        filterCqlStr.append(" and " + layerDataRun);
                    } else {

                        filterCqlStr.append(layerDataRun);

                    }

                }

            }

            System.out.println("查询条件:" + filterCqlStr.toString());

            String host = layerGroupDsView.getHost().split(",")[0].split(":")[0];
            String port = layerGroupDsView.getHost().split(",")[0].split(":")[1];

            Map<String, Object> cqlDsParams = new HashMap<String, Object>();
            cqlDsParams.put(JDBCDataStoreFactory.DATABASE.key, layerGroupDsView.getDataBase());
            cqlDsParams.put(JDBCDataStoreFactory.DBTYPE.key, layerGroupDsView.getDbType());
            cqlDsParams.put(JDBCDataStoreFactory.HOST.key, host);
            cqlDsParams.put(JDBCDataStoreFactory.PORT.key, port);
            cqlDsParams.put(JDBCDataStoreFactory.USER.key, layerGroupDsView.getUserName());
            cqlDsParams.put(JDBCDataStoreFactory.PASSWD.key, layerGroupDsView.getUserPassword());
            cqlDsParams.put("tableName", layerGroupDsView.getTableName());
            Map<String, Object> mapParams = new HashMap<String, Object>();
            if (filterCqlStr.length() > 0) {
                mapParams.put("cql", filterCqlStr.toString());
            }
            PageResult pageResult = ecqlService.cqlQuery(cqlDsParams, mapParams);
            if (pageResult.getRecordCount() > 0) {
                List<Map<String, Object>> cqlDataList = (List<Map<String, Object>>) pageResult.getDatas();
                //3根据空间地理数据生成图层二进制数据
                ShapeUtils.drawImage(requestParam.getBbox(), convert(cqlDataList,layerGroupDsView.getBoxColumn()), layerStyleViewList,g2);

            } else {

                log.info("根据cql规则:" + filterCqlStr.toString() + "和图层代码:" + requestParam.getLayers() + " 没有找到图层对应的数据源的空间地理数据");
            }

        }
        ByteArrayOutputStream buffer = new ByteArrayOutputStream();
        try {
            ImageIO.write(image, "png", buffer);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return buffer.toByteArray();
    }

    public static List<POI> convert(List<Map<String, Object>> maps,String column) {
        List<POI> pois = new ArrayList<>();
        for (Map<String, Object> map : maps) {

            POI poi = new POI();
            poi.setShape(map.get(column).toString());
            pois.add(poi);
        }
        return pois;
}

    @Override
    public List<?> queryElements(String bbox, Integer type, String layerName) {
        return null;
    }

    public static double[] parseBbox(String bboxString) {
        String[] bboxValuesStr = bboxString.split(",");
        if (bboxValuesStr.length != 4) {
            return new double[0]; // 返回空数组表示无效的 BBOX 格式
        }

        double[] bboxValues = new double[4];
        try {
            for (int i = 0; i < 4; i++) {
                bboxValues[i] = Double.parseDouble(bboxValuesStr[i]);
            }
        } catch (NumberFormatException e) {
            return new double[0]; // 返回空数组表示无效的 BBOX 格式
        }

        return bboxValues;
    }

}
